@@ -21,7 +21,7 @@ from pynamodb.constants import (
     BINARY, BINARY_SET, BOOLEAN, DATETIME_FORMAT, DEFAULT_ENCODING,
     LIST, MAP, NULL, NUMBER, NUMBER_SET, STRING, STRING_SET
 )
-from pynamodb.exceptions import AttributeDeserializationError
+from pynamodb.exceptions import AttributeDeserializationError, AttributeNullError
 from pynamodb.expressions.operand import Path
 
 
@@ -163,7 +163,7 @@ class Attribute(Generic[_T]):
     def between(self, lower: Any, upper: Any) -> 'Between':
         return Path(self).between(lower, upper)
 
-    def is_in(self, *values: Any) -> 'In':
+    def is_in(self, *values: _T) -> 'In':
         return Path(self).is_in(*values)
 
     def exists(self) -> 'Exists':
@@ -346,8 +346,12 @@ class AttributeContainer(metaclass=AttributeContainerMeta):
         attribute_values: Dict[str, Dict[str, Any]] = {}
         for name, attr in self.get_attributes().items():
             value = getattr(self, name)
-            if isinstance(value, MapAttribute) and not value.validate(null_check=null_check):
-                raise ValueError("Attribute '{}' is not correctly typed".format(name))
+            try:
+                if isinstance(value, MapAttribute) and not value.validate(null_check=null_check):
+                    raise ValueError("Attribute '{}' is not correctly typed".format(name))
+            except AttributeNullError as e:
+                e.prepend_path(name)
+                raise
 
             if value is not None:
                 if isinstance(attr, MapAttribute):
@@ -357,7 +361,7 @@ class AttributeContainer(metaclass=AttributeContainerMeta):
             else:
                 attr_value = None
             if null_check and attr_value is None and not attr.null:
-                raise ValueError("Attribute '{}' cannot be None".format(name))
+                raise AttributeNullError(name)
 
             if attr_value is not None:
                 attribute_values[attr.attr_name] = {attr.attr_type: attr_value}
@@ -681,7 +685,9 @@ class UTCDateTimeAttribute(Attribute[datetime]):
         """
         if value.tzinfo is None:
             value = value.replace(tzinfo=timezone.utc)
-        fmt = value.astimezone(timezone.utc).strftime(DATETIME_FORMAT)
+        # Padding of years under 1000 is inconsistent and depends on system strftime:
+        # https://bugs.python.org/issue13305
+        fmt = value.astimezone(timezone.utc).strftime(DATETIME_FORMAT).zfill(31)
         return fmt
 
     def deserialize(self, value):
@@ -696,6 +702,8 @@ class UTCDateTimeAttribute(Attribute[datetime]):
         # This is ~5.8x faster than using strptime and 38x faster than dateutil.parser.parse.
         _int = int  # Hack to prevent global lookups of int, speeds up the function ~10%
         try:
+            # Fix pre-1000 dates serialized on systems where strftime doesn't pad w/older PynamoDB versions.
+            date_string = date_string.zfill(31)
             if (len(date_string) != 31 or date_string[4] != '-' or date_string[7] != '-'
                     or date_string[10] != 'T' or date_string[13] != ':' or date_string[16] != ':'
                     or date_string[19] != '.' or date_string[26:31] != '+0000'):
@@ -854,20 +862,20 @@ class MapAttribute(Attribute[Mapping[_KT, _VT]], AttributeContainer):
         # If this instance is being used as an Attribute, treat item access like the map dereference operator.
         # This provides equivalence between DynamoDB's nested attribute access for map elements (MyMap.nestedField)
         # and Python's item access for dictionaries (MyMap['nestedField']).
-        if self.is_raw():
-            return Path(self.attr_path + [str(item)])  # type: ignore
-        elif item in self._attributes:  # type: ignore
+        if item in self.get_attributes():
             return getattr(self, item)
+        elif self.is_raw():
+            return Path(self.attr_path + [str(item)])  # type: ignore
         else:
             raise AttributeError("'{}' has no attribute '{}'".format(self.__class__.__name__, item))
 
     def __setitem__(self, item, value):
         if not self._is_attribute_container():
             raise TypeError("'{}' object does not support item assignment".format(self.__class__.__name__))
-        if self.is_raw():
-            self.attribute_values[item] = value
-        elif item in self._attributes:  # type: ignore
+        if item in self.get_attributes():
             setattr(self, item, value)
+        elif self.is_raw():
+            self.attribute_values[item] = value
         else:
             raise AttributeError("'{}' has no attribute '{}'".format(self.__class__.__name__, item))
 
@@ -918,13 +926,29 @@ class MapAttribute(Attribute[Mapping[_KT, _VT]], AttributeContainer):
         if can_be_null and value is None:
             return True
         if getattr(self, key) is None:
-            raise ValueError("Attribute '{}' cannot be None".format(key))
+            raise AttributeNullError(key)
         return True  # TODO: check that the actual type of `value` meets requirements of `attr`
 
     def validate(self, *, null_check: bool = False):
         return all(self.is_correctly_typed(k, v, null_check=null_check)
                    for k, v in self.get_attributes().items())
 
+    def _serialize_undeclared_attributes(self, values, container: Dict):
+        # Continue to serialize NULL values in "raw" map attributes for backwards compatibility.
+        # This special case behavior for "raw" attributes should be removed in the future.
+        for attr_name in values:
+            if attr_name not in self.get_attributes():
+                v = values[attr_name]
+                attr_class = _get_class_for_serialize(v)
+                attr_type = attr_class.attr_type
+                attr_value = attr_class.serialize(v)
+                if attr_value is None:
+                    # When attribute values serialize to "None" (e.g. empty sets) we store {"NULL": True} in DynamoDB.
+                    attr_type = NULL
+                    attr_value = True
+                container[attr_name] = {attr_type: attr_value}
+        return container
+
     def serialize(self, values, *, null_check: bool = True):
         if not self.is_raw():
             # This is a subclassed MapAttribute that acts as an AttributeContainer.
@@ -941,20 +965,8 @@ class MapAttribute(Attribute[Mapping[_KT, _VT]], AttributeContainer):
 
             return AttributeContainer._container_serialize(values, null_check=null_check)
 
-        # Continue to serialize NULL values in "raw" map attributes for backwards compatibility.
-        # This special case behavior for "raw" attributes should be removed in the future.
-        rval = {}
-        for attr_name in values:
-            v = values[attr_name]
-            attr_class = _get_class_for_serialize(v)
-            attr_type = attr_class.attr_type
-            attr_value = attr_class.serialize(v)
-            if attr_value is None:
-                # When attribute values serialize to "None" (e.g. empty sets) we store {"NULL": True} in DynamoDB.
-                attr_type = NULL
-                attr_value = True
-            rval[attr_name] = {attr_type: attr_value}
-        return rval
+        # For a "raw" MapAttribute all fields are undeclared
+        return self._serialize_undeclared_attributes(values, {})
 
     def deserialize(self, values):
         """
@@ -980,6 +992,62 @@ class MapAttribute(Attribute[Mapping[_KT, _VT]], AttributeContainer):
         return result
 
 
+class DynamicMapAttribute(MapAttribute):
+    """
+    A map attribute that supports declaring attributes (like an AttributeContainer) but will also store
+    any other values that are set on it (like a raw MapAttribute).
+
+    >>> class MyDynamicMapAttribute(DynamicMapAttribute):
+    >>>     a_date_time = UTCDateTimeAttribute()  # raw map attributes cannot serialize/deserialize datetime values
+    >>>
+    >>> dynamic_map = MyDynamicMapAttribute()
+    >>> dynamic_map.a_date_time = datetime.utcnow()
+    >>> dynamic_map.a_number = 5
+    >>> dynamic_map.serialize()  # {'a_date_time': {'S': 'xxx'}, 'a_number': {'N': '5'}}
+    """
+
+    def __setattr__(self, name, value):
+        # Set attributes via the Attribute descriptor if it exists.
+        if name in self.get_attributes():
+            object.__setattr__(self, name, value)
+        else:
+            super().__setattr__(name, value)
+
+    def serialize(self, values, *, null_check: bool = True):
+        if not isinstance(values, type(self)):
+            # Copy the values onto an instance of the class for serialization.
+            instance = type(self)()
+            instance.attribute_values = {}  # clear any defaults
+            instance._set_attributes(**values)
+            values = instance
+
+        # this serializes the class defined attributes.
+        # we do this first because we have type checks that validate the data
+        rval = AttributeContainer._container_serialize(values, null_check=null_check)
+
+        # this serializes the dynamically defined attributes
+        # we have no real type safety here so we have to dynamically construct the type to write to dynamo
+        self._serialize_undeclared_attributes(values, rval)
+
+        return rval
+
+    def deserialize(self, values):
+        # this deserializes the class defined attributes
+        # we do this first so that the we populate the defined object attributes fields properly with type safety
+        instance = self._instantiate(values)
+        # this deserializes the dynamically defined attributes
+        for attr_name, value in values.items():
+            if instance._dynamo_to_python_attr(attr_name) not in instance.get_attributes():
+                attr_type, attr_value = next(iter(value.items()))
+                instance[attr_name] = DESERIALIZE_CLASS_MAP[attr_type].deserialize(attr_value)
+        return instance
+
+    @classmethod
+    def is_raw(cls):
+        # All subclasses of DynamicMapAttribute should be treated like "raw" map attributes.
+        return True
+
+
 def _get_class_for_serialize(value):
     if value is None:
         return NullAttribute()
